@@ -22,7 +22,6 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include "syncres.hh"
 #include "utility.hh"
 #include "lwres.hh"
 #include <iostream>
@@ -39,6 +38,7 @@
 #include "pdnsexception.hh"
 #include "arguments.hh"
 #include "sstuff.hh"
+#include "syncres.hh"
 #include "dnswriter.hh"
 #include "dnsparser.hh"
 #include "logger.hh"
@@ -161,7 +161,7 @@ int asyncresolve(const ComboAddress& ip, const DNSName& domain, int type, bool d
   
     // sleep until we see an answer to this, interface to mtasker
     
-    ret=arecvfrom(reinterpret_cast<char *>(buf.get()), bufsize-1,0, ip, &len, qid,
+    ret=arecvfrom(reinterpret_cast<char *>(buf.get()), bufsize, 0, ip, &len, qid,
                   domain, type, queryfd, now);
   }
   else {
