@@ -27,6 +27,15 @@
 #include <signal.h>
 #include <ucontext.h>
 
+#ifdef PDNS_USE_VALGRIND
+#include <valgrind/valgrind.h>
+#endif /* PDNS_USE_VALGRIND */
+
+#ifdef HAVE_FIBER_SANITIZER
+__thread void* t_mainStack{nullptr};
+__thread size_t t_mainStackSize{0};
+#endif /* HAVE_FIBER_SANITIZER */
+
 template <typename Message> static __attribute__((noinline, cold, noreturn))
 void
 throw_errno (Message&& msg) {
@@ -71,6 +80,7 @@ extern "C" {
 static
 void
 threadWrapper (int const ctx0, int const ctx1, int const fun0, int const fun1) {
+    notifyStackSwitchDone();
     auto ctx = joinPtr<pdns_ucontext_t>(ctx0, ctx1);
     try {
         auto start = std::move(*joinPtr<boost::function<void()>>(fun0, fun1));
@@ -78,16 +88,25 @@ threadWrapper (int const ctx0, int const ctx1, int const fun0, int const fun1) {
     } catch (...) {
         ctx->exception = std::current_exception();
     }
+    notifyStackSwitchToKernel();
 }
 } // extern "C"
 
 pdns_ucontext_t::pdns_ucontext_t() {
     uc_mcontext = new ::ucontext_t();
     uc_link = nullptr;
+#ifdef PDNS_USE_VALGRIND
+    valgrind_id = 0;
+#endif /* PDNS_USE_VALGRIND */
 }
 
 pdns_ucontext_t::~pdns_ucontext_t() {
     delete static_cast<ucontext_t*>(uc_mcontext);
+#ifdef PDNS_USE_VALGRIND
+    if (valgrind_id != 0) {
+      VALGRIND_STACK_DEREGISTER(valgrind_id);
+    }
+#endif /* PDNS_USE_VALGRIND */
 }
 
 void
