@@ -1,8 +1,8 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include "syncres.hh"
 #include "mtasker.hh"
+#include "syncres.hh"
 #include "rec_channel.hh"
 #include "iputils.hh"
 #include "logger.hh"
@@ -36,9 +36,9 @@ try
     boost::replace_all(hostname, ".", "_");    
   }
 
+  registerAllStats();
   string msg;
   for(const auto& carbonServer: carbonServers) {
-    RecursorControlParser rcp; // inits if needed
     ComboAddress remote(carbonServer, 2003);
     Socket s(remote.sin4.sin_family, SOCK_STREAM);
 
@@ -73,3 +73,7 @@ catch(std::exception& e)
 {
   L<<Logger::Error<<"Error in carbon thread: "<<e.what()<<endl;
 }
+catch(...)
+{
+  L<<Logger::Error<<"Unknown error in carbon thread"<<endl;
+}
