@@ -1,20 +1,19 @@
 #include "config.h"
-#ifdef HAVE_LUA
 #include "ext/luawrapper/include/LuaContext.hpp"
-#endif
 
 #include <fstream>
 #include <thread>
 #include "namespaces.hh"
-#include "syncres.hh"
 #include "logger.hh"
 #include "rec-lua-conf.hh"
 #include "sortlist.hh"
 #include "filterpo.hh"
+#include "syncres.hh"
 #include "rpzloader.hh"
 #include "base64.hh"
 #include "remote_logger.hh"
 #include "validate.hh"
+#include "validate-recursor.hh"
 #include "root-dnssec.hh"
 
 GlobalStateHolder<LuaConfigItems> g_luaconfs; 
@@ -34,9 +33,10 @@ GlobalStateHolder<LuaConfigItems> g_luaconfs;
 
 LuaConfigItems::LuaConfigItems()
 {
+  DNSName root("."); // don't use g_rootdnsname here, it might not exist yet
   for (const auto &dsRecord : rootDSs) {
-    auto ds=std::dynamic_pointer_cast<DSRecordContent>(DSRecordContent::make(dsRecord));
-    dsAnchors[DNSName(".")].insert(*ds);
+    auto ds=unique_ptr<DSRecordContent>(dynamic_cast<DSRecordContent*>(DSRecordContent::make(dsRecord)));
+    dsAnchors[root].insert(*ds);
   }
 }
 
@@ -51,13 +51,35 @@ typename C::value_type::second_type constGet(const C& c, const std::string& name
   return iter->second;
 }
 
-#ifndef HAVE_LUA
-void loadRecursorLuaConfig(const std::string& fname, bool checkOnly)
+
+static void parseRPZParameters(const std::unordered_map<string,boost::variant<uint32_t, string> >& have, std::string& polName, boost::optional<DNSFilterEngine::Policy>& defpol, uint32_t& maxTTL, size_t& zoneSizeHint)
 {
-  if(!fname.empty())
-    throw PDNSException("Asked to load a Lua configuration file '"+fname+"' in binary without Lua support");
+  if(have.count("policyName")) {
+    polName = boost::get<std::string>(constGet(have, "policyName"));
+  }
+  if(have.count("defpol")) {
+    defpol=DNSFilterEngine::Policy();
+    defpol->d_kind = (DNSFilterEngine::PolicyKind)boost::get<uint32_t>(constGet(have, "defpol"));
+    defpol->d_name = std::make_shared<std::string>(polName);
+    if(defpol->d_kind == DNSFilterEngine::PolicyKind::Custom) {
+      defpol->d_custom=
+          DNSRecordContent::mastermake(QType::CNAME, 1,
+                                       boost::get<string>(constGet(have,"defcontent"))
+            );
+
+      if(have.count("defttl"))
+        defpol->d_ttl = static_cast<int32_t>(boost::get<uint32_t>(constGet(have, "defttl")));
+      else
+        defpol->d_ttl = -1; // get it from the zone
+    }
+  }
+  if(have.count("maxTTL")) {
+    maxTTL = boost::get<uint32_t>(constGet(have, "maxTTL"));
+  }
+  if(have.count("zoneSizeHint")) {
+    zoneSizeHint = static_cast<size_t>(boost::get<uint32_t>(constGet(have, "zoneSizeHint")));
+  }
 }
-#else
 
 void loadRecursorLuaConfig(const std::string& fname, bool checkOnly)
 {
@@ -87,121 +109,91 @@ void loadRecursorLuaConfig(const std::string& fname, bool checkOnly)
   };
   Lua.writeVariable("Policy", pmap);
 
-  Lua.writeFunction("rpzFile", [&lci](const string& filename, const boost::optional<std::unordered_map<string,boost::variant<int, string>>>& options) {
+  Lua.writeFunction("rpzFile", [&lci](const string& filename, const boost::optional<std::unordered_map<string,boost::variant<uint32_t, string>>>& options) {
       try {
-	boost::optional<DNSFilterEngine::Policy> defpol;
-	std::string polName("rpzFile");
-	const size_t zoneIdx = lci.dfe.size();
-	if(options) {
-	  auto& have = *options;
-	  if(have.count("policyName")) {
-	    polName = boost::get<std::string>(constGet(have, "policyName"));
-	  }
-	  if(have.count("defpol")) {
-	    defpol=DNSFilterEngine::Policy();
-	    defpol->d_kind = (DNSFilterEngine::PolicyKind)boost::get<int>(constGet(have, "defpol"));
-	    defpol->d_name = std::make_shared<std::string>(polName);
-	    if(defpol->d_kind == DNSFilterEngine::PolicyKind::Custom) {
-	      defpol->d_custom=
-		shared_ptr<DNSRecordContent>(
-					     DNSRecordContent::mastermake(QType::CNAME, 1, 
-									  boost::get<string>(constGet(have,"defcontent"))
-									  )
-					     );
-	 
-	      if(have.count("defttl"))
-		defpol->d_ttl = boost::get<int>(constGet(have, "defttl"));
-	      else
-		defpol->d_ttl = -1; // get it from the zone
-	    }
-	  }
-          if(have.count("zoneSizeHint")) {
-            lci.dfe.reserve(zoneIdx, static_cast<size_t>(boost::get<int>(constGet(have, "zoneSizeHint"))));
+        boost::optional<DNSFilterEngine::Policy> defpol;
+        std::string polName("rpzFile");
+        std::shared_ptr<DNSFilterEngine::Zone> zone = std::make_shared<DNSFilterEngine::Zone>();
+        uint32_t maxTTL = std::numeric_limits<uint32_t>::max();
+        if(options) {
+          auto& have = *options;
+          size_t zoneSizeHint = 0;
+          parseRPZParameters(have, polName, defpol, maxTTL, zoneSizeHint);
+          if (zoneSizeHint > 0) {
+            zone->reserve(zoneSizeHint);
           }
-	}
+        }
         theL()<<Logger::Warning<<"Loading RPZ from file '"<<filename<<"'"<<endl;
-        lci.dfe.setPolicyName(zoneIdx, polName);
-        loadRPZFromFile(filename, lci.dfe, defpol, zoneIdx);
+        zone->setName(polName);
+        loadRPZFromFile(filename, zone, defpol, maxTTL);
+        lci.dfe.addZone(zone);
         theL()<<Logger::Warning<<"Done loading RPZ from file '"<<filename<<"'"<<endl;
       }
-      catch(std::exception& e) {
-	theL()<<Logger::Error<<"Unable to load RPZ zone from '"<<filename<<"': "<<e.what()<<endl;
+      catch(const std::exception& e) {
+        theL()<<Logger::Error<<"Unable to load RPZ zone from '"<<filename<<"': "<<e.what()<<endl;
       }
     });
 
-
-  Lua.writeFunction("rpzMaster", [&lci, checkOnly](const string& master_, const string& zone_, const boost::optional<std::unordered_map<string,boost::variant<int, string>>>& options) {
+  Lua.writeFunction("rpzMaster", [&lci, checkOnly](const string& master_, const string& zoneName, const boost::optional<std::unordered_map<string,boost::variant<uint32_t, string>>>& options) {
       try {
-	boost::optional<DNSFilterEngine::Policy> defpol;
+        boost::optional<DNSFilterEngine::Policy> defpol;
+        std::shared_ptr<DNSFilterEngine::Zone> zone = std::make_shared<DNSFilterEngine::Zone>();
         TSIGTriplet tt;
-        int refresh=0;
-	std::string polName;
-	size_t maxReceivedXFRMBytes = 0;
-	ComboAddress localAddress;
-	const size_t zoneIdx = lci.dfe.size();
-	if(options) {
-	  auto& have = *options;
-          polName = zone_;
-	  if(have.count("policyName"))
-	    polName = boost::get<std::string>(constGet(have, "policyName"));
-	  if(have.count("defpol")) {
-	    //	    cout<<"Set a default policy"<<endl;
-	    defpol=DNSFilterEngine::Policy();
-	    defpol->d_kind = (DNSFilterEngine::PolicyKind)boost::get<int>(constGet(have, "defpol"));
-	    defpol->d_name = std::make_shared<std::string>(polName);
-	    if(defpol->d_kind == DNSFilterEngine::PolicyKind::Custom) {
-	      //	      cout<<"Setting a custom field even!"<<endl;
-	      defpol->d_custom=
-		shared_ptr<DNSRecordContent>(
-					     DNSRecordContent::mastermake(QType::CNAME, 1, 
-									  boost::get<string>(constGet(have,"defcontent"))
-									  )
-					     );
-	      if(have.count("defttl"))
-		defpol->d_ttl = boost::get<int>(constGet(have, "defttl"));
-	      else
-		defpol->d_ttl = -1; // get it from the zone
-	    }
-	  }
-          if(have.count("zoneSizeHint")) {
-            lci.dfe.reserve(zoneIdx, static_cast<size_t>(boost::get<int>(constGet(have, "zoneSizeHint"))));
+        uint32_t refresh=0;
+        std::string polName(zoneName);
+        size_t maxReceivedXFRMBytes = 0;
+        uint32_t maxTTL = std::numeric_limits<uint32_t>::max();
+        ComboAddress localAddress;
+        if(options) {
+          auto& have = *options;
+          size_t zoneSizeHint = 0;
+          parseRPZParameters(have, polName, defpol, maxTTL, zoneSizeHint);
+          if (zoneSizeHint > 0) {
+            zone->reserve(zoneSizeHint);
           }
-	  if(have.count("tsigname")) {
+          if(have.count("tsigname")) {
             tt.name=DNSName(toLower(boost::get<string>(constGet(have, "tsigname"))));
             tt.algo=DNSName(toLower(boost::get<string>(constGet(have, "tsigalgo"))));
             if(B64Decode(boost::get<string>(constGet(have, "tsigsecret")), tt.secret))
               throw std::runtime_error("TSIG secret is not valid Base-64 encoded");
           }
           if(have.count("refresh")) {
-            refresh = boost::get<int>(constGet(have,"refresh"));
+            refresh = boost::get<uint32_t>(constGet(have,"refresh"));
           }
           if(have.count("maxReceivedMBytes")) {
-            maxReceivedXFRMBytes = static_cast<size_t>(boost::get<int>(constGet(have,"maxReceivedMBytes")));
+            maxReceivedXFRMBytes = static_cast<size_t>(boost::get<uint32_t>(constGet(have,"maxReceivedMBytes")));
           }
           if(have.count("localAddress")) {
             localAddress = ComboAddress(boost::get<string>(constGet(have,"localAddress")));
           }
-	}
-	ComboAddress master(master_, 53);
-        if (localAddress != ComboAddress() && localAddress.sin4.sin_family != master.sin4.sin_family)
+        }
+        ComboAddress master(master_, 53);
+        if (localAddress != ComboAddress() && localAddress.sin4.sin_family != master.sin4.sin_family) {
           // We were passed a localAddress, check if its AF matches the master's
           throw PDNSException("Master address("+master.toString()+") is not of the same Address Family as the local address ("+localAddress.toString()+").");
-	DNSName zone(zone_);
-        lci.dfe.setPolicyName(zoneIdx, polName);
+        }
+
+        DNSName domain(zoneName);
+        zone->setDomain(domain);
+        zone->setName(polName);
+        zone->setRefresh(refresh);
+        size_t zoneIdx = lci.dfe.addZone(zone);
 
         if (!checkOnly) {
-          auto sr=loadRPZFromServer(master, zone, lci.dfe, defpol, zoneIdx, tt, maxReceivedXFRMBytes * 1024 * 1024, localAddress);
+          auto sr=loadRPZFromServer(master, domain, zone, defpol, maxTTL, tt, maxReceivedXFRMBytes * 1024 * 1024, localAddress);
           if(refresh)
             sr->d_st.refresh=refresh;
-          std::thread t(RPZIXFRTracker, master, zone, defpol, zoneIdx, tt, sr, maxReceivedXFRMBytes * 1024 * 1024, localAddress);
+          zone->setSerial(sr->d_st.serial);
+
+          std::thread t(RPZIXFRTracker, master, DNSName(zoneName), defpol, maxTTL, zoneIdx, tt, sr, maxReceivedXFRMBytes * 1024 * 1024, localAddress);
           t.detach();
         }
       }
-      catch(std::exception& e) {
-	theL()<<Logger::Error<<"Unable to load RPZ zone '"<<zone_<<"' from '"<<master_<<"': "<<e.what()<<endl;
+      catch(const std::exception& e) {
+        theL()<<Logger::Error<<"Unable to load RPZ zone '"<<zoneName<<"' from '"<<master_<<"': "<<e.what()<<endl;
       }
-      catch(PDNSException& e) {
-	theL()<<Logger::Error<<"Unable to load RPZ zone '"<<zone_<<"' from '"<<master_<<"': "<<e.reason<<endl;
+      catch(const PDNSException& e) {
+        theL()<<Logger::Error<<"Unable to load RPZ zone '"<<zoneName<<"' from '"<<master_<<"': "<<e.reason<<endl;
       }
 
     });
@@ -239,12 +231,14 @@ void loadRecursorLuaConfig(const std::string& fname, bool checkOnly)
 		    });
 
   Lua.writeFunction("addDS", [&lci](const std::string& who, const std::string& what) {
+      warnIfDNSSECDisabled("Warning: adding Trust Anchor for DNSSEC (addDS), but dnssec is set to 'off'!");
       DNSName zone(who);
-      auto ds=std::dynamic_pointer_cast<DSRecordContent>(DSRecordContent::make(what));
+      auto ds = unique_ptr<DSRecordContent>(dynamic_cast<DSRecordContent*>(DSRecordContent::make(what)));
       lci.dsAnchors[zone].insert(*ds);
   });
 
   Lua.writeFunction("clearDS", [&lci](boost::optional<string> who) {
+      warnIfDNSSECDisabled("Warning: removing Trust Anchor for DNSSEC (clearDS), but dnssec is set to 'off'!");
       if(who)
         lci.dsAnchors.erase(DNSName(*who));
       else
@@ -252,6 +246,7 @@ void loadRecursorLuaConfig(const std::string& fname, bool checkOnly)
     });
 
   Lua.writeFunction("addNTA", [&lci](const std::string& who, const boost::optional<std::string> why) {
+      warnIfDNSSECDisabled("Warning: adding Negative Trust Anchor for DNSSEC (addNTA), but dnssec is set to 'off'!");
       if(why)
         lci.negAnchors[DNSName(who)] = static_cast<string>(*why);
       else
@@ -259,6 +254,7 @@ void loadRecursorLuaConfig(const std::string& fname, bool checkOnly)
     });
 
   Lua.writeFunction("clearNTA", [&lci](boost::optional<string> who) {
+      warnIfDNSSECDisabled("Warning: removing Negative Trust Anchor for DNSSEC (clearNTA), but dnssec is set to 'off'!");
       if(who)
         lci.negAnchors.erase(DNSName(*who));
       else
@@ -266,7 +262,7 @@ void loadRecursorLuaConfig(const std::string& fname, bool checkOnly)
     });
 
 #if HAVE_PROTOBUF
-  Lua.writeFunction("protobufServer", [&lci, checkOnly](const string& server_, const boost::optional<uint16_t> timeout, const boost::optional<uint64_t> maxQueuedEntries, const boost::optional<uint8_t> reconnectWaitTime, const boost::optional<uint8_t> maskV4, boost::optional<uint8_t> maskV6, boost::optional<bool> asyncConnect) {
+  Lua.writeFunction("protobufServer", [&lci, checkOnly](const string& server_, const boost::optional<uint16_t> timeout, const boost::optional<uint64_t> maxQueuedEntries, const boost::optional<uint8_t> reconnectWaitTime, const boost::optional<uint8_t> maskV4, boost::optional<uint8_t> maskV6, boost::optional<bool> asyncConnect, boost::optional<bool> taggedOnly) {
       try {
 	ComboAddress server(server_);
         if (!lci.protobufServer) {
@@ -280,6 +276,9 @@ void loadRecursorLuaConfig(const std::string& fname, bool checkOnly)
           if (maskV6) {
             lci.protobufMaskV6 = *maskV6;
           }
+          if (taggedOnly) {
+            lci.protobufTaggedOnly = *taggedOnly;
+          }
         }
         else {
           theL()<<Logger::Error<<"Only one protobuf server can be configured, we already have "<<lci.protobufServer->toString()<<endl;
@@ -340,4 +339,3 @@ void loadRecursorLuaConfig(const std::string& fname, bool checkOnly)
 
 }
 
-#endif
