@@ -1,3 +1,6 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "utility.hh"
 #include <cstdio>
 #include <math.h>
@@ -134,6 +137,9 @@ latlon2ul(const char **latlonstrptr, int *which)
     break;
   }
 
+  if (!*cp)
+    return 0;
+
   cp++;                   /* skip the hemisphere */
   
   while (*cp && !isspace(*cp))   /* if any trailing garbage */
@@ -186,7 +192,7 @@ LOCRecordContent::DNSRecordContent* LOCRecordContent::make(const DNSRecord &dr,
   return ret;
 }
 
-LOCRecordContent::LOCRecordContent(const string& content, const string& zone) : DNSRecordContent(QType::LOC)
+LOCRecordContent::LOCRecordContent(const string& content, const string& zone) 
 {
   // 51 59 00.000 N 5 55 00.000 E 4.00m 1.00m 10000.00m 10.00m
   // convert this to d_version, d_size, d_horiz/vertpre, d_latitude, d_longitude, d_altitude
@@ -287,7 +293,7 @@ LOCRecordContent::LOCRecordContent(const string& content, const string& zone) :
 }
 
 
-string LOCRecordContent::getZoneRepresentation() const
+string LOCRecordContent::getZoneRepresentation(bool noDot) const
 {
   // convert d_version, d_size, d_horiz/vertpre, d_latitude, d_longitude, d_altitude to:
   // 51 59 00.000 N 5 55 00.000 E 4.00m 1.00m 10000.00m 10.00m
